from napoleontoolbox.rebalancing import rolling
from sklearn.metrics import mean_squared_error
from sklearn.metrics import accuracy_score
from sklearn.metrics import confusion_matrix
import pandas as pd

def rolling_forecasting(forecasting_model, X, y, n=252, s=63, method = 'standard', display = False, **kwargs):
    assert X.shape[0] == y.shape[0]
    idx = X.index
    forecasting_series = pd.Series(index=idx, name='prediction')
    roll = rolling._RollingMechanism(idx, n=n, s=s)
    for slice_n, slice_s in roll():
        # Select X
        X_train = X.loc[slice_n].copy()
        y_train = y.loc[slice_n].copy()

        X_test = X.loc[slice_s].copy()
        y_test = y.loc[slice_s].copy()

        forecasting_model.fit(X_train, y_train, method)
        y_pred = forecasting_model.predict(X_test, method)
        forecasting_series.loc[slice_s] = y_pred

        matrix = confusion_matrix(y_test > 0, y_pred > 0)
        rmse = mean_squared_error(y_test, y_pred)
        accuracy = accuracy_score(y_test > 0, y_pred > 0)

        if display:
            print('rmse for slice ' + str(slice_s) +' : '+str(rmse))
            print('accuracy')
            print(accuracy)
            print('confusion matrix')
            print(matrix)

    return forecasting_series




from sklearn.ensemble import RandomForestRegressor

class ForecasterWrapper:
    """ Base object to roll a neural network model.
    Rolling over a time axis with a train period from `t - n` to `t` and a
    testing period from `t` to `t + s`.
    Parameters
    ----------
    X, y : array_like
        Respectively input and output data.
    f : callable, optional
        Function to transform target, e.g. ``torch.sign`` function.
    index : array_like, optional
        Time index of data.
    Methods
    -------
    __call__
    Attributes
    ----------
    n, s, r : int
        Respectively size of training, testing and rolling period.
    b, e, T : int
        Respectively batch size, number of repass_steps and size of entire dataset.
    t : int
        The current time period.
    y_eval, y_test : np.ndarray[ndim=1 or 2, dtype=np.float64]
        Respectively evaluating (or training) and testing predictions.
    """
    def __init__(self, method = 'standard'):
        """ Initialize shape of target. """
        if method == 'standard':
            self.model  = RandomForestRegressor(max_depth=5, random_state=0)

    def fit(self, X_train, y_train, method = 'standard'):
        if method == 'standard':
            self.model.fit(X_train, y_train)

    def predict(self, X_test, method = 'standard'):
        if method == 'standard':
            y_pred = self.model.predict(X_test)
            return y_pred
        return None




